/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: SimpleObservation.java,v 3e7ea5dbefcb 2010/05/17 15:10:54 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.observation.simple;

import javax.annotation.Nonnull;
import java.util.Set;
import java.util.Date;
import java.io.Serializable;
import it.tidalwave.bluebill.observation.Finder;
import it.tidalwave.bluebill.observation.Location;
import it.tidalwave.bluebill.observation.Observation;
import it.tidalwave.bluebill.observation.ObservationItem;
import it.tidalwave.bluebill.observation.Observer;
import it.tidalwave.bluebill.observation.Source;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class SimpleObservation implements Observation, Serializable
  {
    private static final long serialVersionUID = 56354537593487L;

    @Nonnull
    private final SimpleObservationSet observationSet;

    @Nonnull
    private final Date date;

    @Nonnull
    private final Location location;

    @Nonnull
    private final Set<Observer> observers;

    @Nonnull
    private final Set<ObservationItem> observationItems;

    @Nonnull
    private final String note;

    @Nonnull
    private final Source origin;

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public SimpleObservation (final @Nonnull SimpleObservationSet observationSet,
                              final @Nonnull Date date,
                              final @Nonnull Location location,
                              final @Nonnull Set<Observer> observers,
                              final @Nonnull Set<ObservationItem> observationItems,
                              final @Nonnull String note,
                              final @Nonnull Source origin)
      {
        this.observationSet = observationSet;
        this.date = date;
        this.location = location;
        this.observers = observers;
        this.observationItems = observationItems;
        this.note = note;
        this.origin = origin;
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Date getDate()
      {
        return new Date(date.getTime());
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Location getLocation()
      {
        return location;
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Finder<ObservationItem> findObservationItems()
      {
        return new SimpleFinderSupport<ObservationItem>(observationItems);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public Finder<Observer> findObservers()
      {
        return new SimpleFinderSupport<Observer>(observers);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    public ReplaceBuilder replace()
      {
        return new SimpleObservationReplaceBuilder(observationSet, this);
      }
  }
