/***********************************************************************************************************************
 *
 * blueBill Mobile - open source birdwatching
 * ==========================================
 *
 * Copyright (C) 2009, 2010 by Tidalwave s.a.s. (http://www.tidalwave.it)
 * http://bluebill.tidalwave.it/mobile/
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id: SimpleObservationBaseBuilder.java,v 3e7ea5dbefcb 2010/05/17 15:10:54 fabrizio $
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.observation.simple;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import it.tidalwave.util.Initializer;
import it.tidalwave.bluebill.observation.Cardinality;
import it.tidalwave.bluebill.observation.Location;
import it.tidalwave.bluebill.observation.Observable;
import it.tidalwave.bluebill.observation.Observation;
import it.tidalwave.bluebill.observation.ObservationItem;
import it.tidalwave.bluebill.observation.Observer;
import it.tidalwave.bluebill.observation.Source;
import javax.annotation.Nonnegative;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: SimpleObservationBaseBuilder.java,v 3e7ea5dbefcb 2010/05/17 15:10:54 fabrizio $
 *
 ******************************************************************************/
public class SimpleObservationBaseBuilder implements Observation.BaseBuilder, Observation.Builder
  {
    /***************************************************************************
     *
     *
     **************************************************************************/
    protected static class ObservableWithCardinality
      {
        @Nonnull
        public final Observable observable;

        @Nonnull
        public final Cardinality cardinality;

        private ObservableWithCardinality (@Nonnull final Observable observable,
                                           @Nonnull final Cardinality cardinality)
          {
            this.observable = observable;
            this.cardinality = cardinality;
          }

        @Override
        @Nonnull
        public String toString()
          {
            return String.format("%s x %s", cardinality, observable);
          }
      }
    
    @Nonnull
    protected final SimpleObservationSet observationSet;

    protected Date date;

    protected Location location;

    protected String note;

    protected Source origin;

    protected final Set<Observer> observers = new HashSet<Observer>();

    protected final List<ObservableWithCardinality> observablesWithCardinality = new ArrayList<ObservableWithCardinality>();
    
    protected final Map<ObservableWithCardinality, Initializer<ObservationItem>> initializerMap =
            new HashMap<ObservableWithCardinality, Initializer<ObservationItem>>();

    /***************************************************************************
     *
     *
     **************************************************************************/
    protected SimpleObservationBaseBuilder (@Nonnull final SimpleObservationSet observationSet)
      {
        this.observationSet = observationSet;
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Nonnull
    public Observation.Builder date (@Nonnull final Date date)
      {
        this.date = date;
        return this;
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Nonnull
    public Observation.Builder location (@Nonnull final Location location)
      {
        this.location = location;
        return this;
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull
    public Observation.Builder item (@Nonnull final Observable observable,
                                       @Nonnull final Cardinality count)
      {
        return item(observable, count, null);
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Nonnull
    public Observation.Builder item (@Nonnull final Observable observable,
                                     @Nonnull final Cardinality cardinality,
                                     @Nonnull final Initializer<ObservationItem> initializer)
      {
        final ObservableWithCardinality observableWithCardinality = new ObservableWithCardinality(observable, cardinality);
        observablesWithCardinality.add(observableWithCardinality);

        if (initializer != null)
          {
            initializerMap.put(observableWithCardinality, initializer);
          }
        
        return this;
      }

    @Nonnegative
    public int getItemCount()
      {
        return observablesWithCardinality.size();
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Nonnull
    public Observation.Builder observer (@Nonnull final Observer observer)
      {
        observers.add(observer);
        return this;
      }

    /***************************************************************************
     *
     * {@inheritDoc}
     *
     **************************************************************************/
    @Nonnull
    public Observation.Builder source (@Nonnull final Source origin)
      {
        this.origin = origin;
        return this;
      }

    /***************************************************************************
     *
     *
     **************************************************************************/
    @Nonnull
    public Observation.Builder note (@Nonnull final String note)
      {
        this.note = note;
        return this;
      }

    public Observation build()
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }

    public Observation apply()
      {
        throw new UnsupportedOperationException("Not supported yet.");
      }
  }
